from openpyxl import load_workbook
from openpyxl.worksheet.datavalidation import DataValidation
from webui_frame.commons.kdt import Word

attr_list = [attr for attr in dir(Word) if not attr.startswith('_')]
attr_list = sorted(attr_list, key=len)
attr_list.insert(0, 'mark')
attr_list.insert(0, 'name')

excel = "webui_frame/tests/test_conv.xlsx"
wb = load_workbook(excel)
ws = wb.active

dv = DataValidation(type="list", formula1=f'"{",".join(attr_list)}"')
ws.add_data_validation(dv)
dv.add('B2:B20')
wb.save(excel)



import argparse
import os
import shutil
from pathlib import Path

base_path = Path(__file__).resolve().parent


def create_project(project_dir, project_type):
    if os.path.exists(project_dir):
        print(f"Warning: {project_dir} already exists.")
        return
    match project_type:
        case "1":
            shutil.copytree(str(base_path / "webui_frame"), project_dir)
        case "2":
            print("api frame not supported now ")
        case "3":
            print("appui frame not supported now ")
        case _:
            print(f"Error: {project_type} not supported")
    if os.path.exists(project_dir):
        print(f"test project {project_dir} created successfully!")


def main():
    parser = argparse.ArgumentParser(description="Autotest init tool")
    parser.add_argument("name", help="project name")
    parser.add_argument(
        "-t",
        "--type",
        choices=["1", "2", "3"],
        required=True,
        help="test type:\n"
             "1、webui test。\n"
             "2、api test。\n"
             "3、appui test。",
    )

    args = parser.parse_args()
    project_dir = os.path.join(os.getcwd(), args.name)
    create_project(project_dir, args.type)



import logging

import allure
import pytest
from commons.kdt import Word

logger = logging.getLogger(__name__)


@pytest.fixture
def login_page(page):
    wd = Word(page)
    """
    自定义fixture实现登录逻辑
    wd.goto("https://online.hitpaw.com/tools/v2/account.html#/login")
    wd.fill('//*[@id="loginForm_email"]', "test@hitpaw.com")
    wd.fill('//*[@id="loginForm_password"]', value="password123")
    wd.click('//*[@id="loginForm"]/div[4]/div/div/div/button')
    wd.assert_contain_text("//html/body/div[1]/div/div/div/div/div/span[2]","Login successful")
    """
    return page


@pytest.fixture(scope="session")
def browser_context_args(browser_context_args):
    return {
        **browser_context_args,
        "viewport": {
            "width": 1920,
            "height": 1080,
        },
    }


def pytest_xlsx_run_step(item):
    for k, v in item.usefixtures.items():
        if "page" in k:
            page = v
            break
    else:
        raise RuntimeError("fixture 'page' is missing")
    wd = Word(page)
    step = item.current_step
    key_name = step["标记"]
    key_args = []
    for k, v in step.items():
        if "列" in k and v:
            key_args.append(v)
    else:
        logger.info(f"{key_name=},{key_args=}")
        func = getattr(wd, key_name)
        func(*key_args)
    # 自动截图
    allure.attach(page.screenshot(), f"{step['标记']}_{step['列1']}")
    return True



from playwright.sync_api import Page, expect


class Word:
    """
    locator操作
    https://playwright.dev/python/docs/actionability
    """

    def __init__(self, page: Page):
        self.page = page

    def __getattr__(self, item):
        if item.startswith("assert_"):
            if item not in self.__dict__:
                self._assert_func = item.replace("assert_", "")
                return self._assert

    def _assert(self, *args):
        """
        支持所有locator所有断言
        https://playwright.dev/python/docs/test-assertions
        eg: assert_to_have_text(selector,expected)
        """
        exp = expect(self.page.locator(args[0]))
        getattr(exp, self._assert_func)(*args[1:])

    def goto(self, url):
        """页面跳转"""
        self.page.goto(url)

    def click(self, selector, click_count=1):
        """点击元素"""
        self.page.click(selector, click_count=click_count)

    def fill(self, selector, value):
        """输入内容"""
        self.page.fill(selector, value)

    def set_input_files(self, selector, files):
        """上传文件<input type="file">"""
        self.page.set_input_files(selector, files)

    def wait_for_selector(self, selector, timeout=10):
        """等待selector"""
        self.page.wait_for_selector(selector, timeout=timeout * 1000)

    def wait_for_timeout(self, timeout):
        """固定等待"""
        self.page.wait_for_timeout(timeout=timeout * 1000)

    def evaluate(self, expression):
        """执行js脚本"""
        return self.page.evaluate(expression)

    def assert_contain_text(self, selector, expected):
        """断言selector包含文本"""
        expect(self.page.locator(selector)).to_contain_text(expected)

    def assert_not_contain_text(self, selector, expected):
        """断言selector不包含文本"""
        expect(self.page.locator(selector)).not_to_contain_text(expected)

    def assert_have_attribute(self, selector, name, value):
        """断言selector有属性值"""
        expect(self.page.locator(selector)).to_have_attribute(name, value)

    def assert_not_have_attribute(self, selector, name, value):
        """断言selector没有属性值"""
        expect(self.page.locator(selector)).not_to_have_attribute(name, value)
